@@ -6,11 +6,11 @@ use strictures 1;
 use utf8;
 
 use Moo;
-with 'MooX::Log::Any', 'Boxer::Role::Interact';
+with 'MooX::Role::Logger', 'Boxer::Role::Interact';
 
 use Role::Commons -all;
 
 our $AUTHORITY = 'cpan:JONASS';
-our $VERSION   = '0.003';
+our $VERSION   = 'v1.1.0';
 
 1;
