@@ -4,11 +4,13 @@ use 5.010;
 use autodie;
 use strictures 1;
 use utf8;
+use Carp qw<croak>;
 
 use Moo;
 extends 'Boxer::Task';
 
-use Types::Standard qw( HashRef Str Undef );
+use Boxer::Types qw( SkelDir Suite );
+use Types::Standard qw( Bool HashRef Str Undef );
 use Types::Path::Tiny qw( Dir File Path );
 use Path::Tiny;
 use Try::Tiny;
@@ -18,7 +20,7 @@ use File::ShareDir::ProjectDistDir;
 use Role::Commons -all;
 
 our $AUTHORITY = 'cpan:JONASS';
-our $VERSION   = '0.004';
+our $VERSION   = 'v1.1.0';
 
 # permit callers to sloppily pass undefined hash values
 sub BUILDARGS
@@ -36,8 +38,8 @@ has data => (
 
 has skeldir => (
 	is       => 'ro',
-	isa      => Dir,
-	coerce   => Dir->coercion,
+	isa      => SkelDir,
+	coerce   => 1,
 	required => 1,
 	default  => sub { Path::Tiny->new( dist_dir('Boxer') )->child('skel') },
 );
@@ -88,6 +90,21 @@ has node => (
 	required => 1,
 );
 
+has nonfree => (
+	is       => 'ro',
+	isa      => Bool,
+	required => 1,
+	default  => sub {0},
+);
+
+has suite => (
+	is       => 'ro',
+	isa      => Suite,
+	required => 1,
+	coerce   => 1,
+	default  => sub {'wheezy'},
+);
+
 my $pos           = 1;
 my @section_order = qw(
 	Administration
@@ -106,7 +123,7 @@ sub run
 	my $self = shift;
 
 	( defined( $self->data->{'nodes'}{ $self->node } ) )
-		or die "Undefined node \"" . $self->node . "\".";
+		or croak "Undefined node \"" . $self->node . "\".";
 
 	my %params = %{ $self->data->{'nodes'}{ $self->node }{'parameters'} };
 
@@ -119,10 +136,22 @@ sub run
 
 	foreach my $key (@section_keys) {
 		my $headline = $params{doc}{$key}{headline}[0] || $key;
-		if ( $params{pkg} and $params{doc}{$key}{pkg} ) {
+		if (( $params{pkg} and $params{doc}{$key}{pkg} )
+			or (    $self->nonfree
+				and $params{'pkg-nonfree'}
+				and $params{doc}{$key}{'pkg-nonfree'} )
+			)
+		{
 			push @{ $desc{pkg} }, "# $headline";
-			foreach ( @{ $params{doc}{$key}{pkg} } ) {
-				push @{ $desc{pkg} }, "#  * $_";
+			if ( $params{pkg} ) {
+				foreach ( @{ $params{doc}{$key}{pkg} } ) {
+					push @{ $desc{pkg} }, "#  * $_";
+				}
+			}
+			if ( $self->nonfree and $params{'pkg-nonfree'} ) {
+				foreach ( @{ $params{doc}{$key}{'pkg-nonfree'} } ) {
+					push @{ $desc{pkg} }, "#  * [non-free] $_";
+				}
 			}
 		}
 		if ( $params{tweak} and $params{doc}{$key}{tweak} ) {
@@ -140,13 +169,31 @@ sub run
 		= defined( $desc{tweak} )
 		? join( "\n", @{ $desc{tweak} } )
 		: '';
-	my @pkg = try { @{ $params{pkg} } } catch { die "No packages resolved" };
+	my @pkg = try { @{ $params{pkg} } }
+	catch {
+		$self->_logger->warning('No packages resolved');
+		return ();
+	};
 	my @pkgauto = try { @{ $params{'pkg-auto'} } }
-	catch { warn "No package auto-markings resolved" };
+	catch {
+		$self->_logger->warning('No package auto-markings resolved');
+		return ();
+	};
 	my @pkgavoid = try { @{ $params{'pkg-avoid'} } }
-	catch { warn "No package avoidance resolved" };
-	my @tweak
-		= try { @{ $params{tweak} } } catch { warn "No tweaks resolved" };
+	catch {
+		$self->_logger->warning('No package avoidance resolved');
+		return ();
+	};
+	my @tweak = try { @{ $params{tweak} } }
+	catch {
+		$self->_logger->warning('No tweaks resolved');
+		return ();
+	};
+	if ( $self->nonfree ) {
+		push @pkg, @{ $params{'pkg-nonfree'} if ( $params{'pkg-nonfree'} ) };
+		push @pkgauto, @{ $params{'pkg-nonfree-auto'} }
+			if ( $params{'pkg-nonfree-auto'} );
+	}
 	my $pkglist = join( ' ', sort @pkg );
 	$pkglist .= " \\\n ";
 	$pkglist .= join( ' ', sort map { $_ . '-' } @pkgavoid );
@@ -161,15 +208,19 @@ sub run
 	);
 
 	my %vars = (
+		node        => $self->node,
+		suite       => $self->suite,
 		pkgdesc     => $pkgdesc,
 		pkglist     => $pkglist,
 		tweakdesc   => $tweakdesc,
 		tweaklist   => $tweaklist,
 		pkgautolist => $pkgautolist,
 	);
+
 	# TODO: drop this when fixed boxer-data is in common use
 	if ( $vars{tweaklist} =~ s,__PKGAUTOLIST__,$pkgautolist, ) {
-		warn "Embedding __PKGAUTOLIST__ in tweaks is deprecated";
+		$self->_logger->warning(
+			'Embedding __PKGAUTOLIST__ in tweaks is deprecated');
 		$vars{pkgautolist} = undef;
 	}
 
@@ -177,6 +228,9 @@ sub run
 	$altvars{tweaklist} =~ s,chroot\s+/target\s+,,g;
 	$altvars{tweaklist} =~ s,/target/,/,g;
 
+	# TODO: maybe move below (or only $''{ part?) to reclass parser
+	$altvars{tweaklist} =~ s/\\\K''(?=n)|\$\K''(?=\{)//g;
+
 	my $altcontent = '';
 	$template->process( \$self->altinfile->slurp, \%altvars, \$altcontent );
 	$self->altoutfile->spew( $altcontent . "\n" );
